#
# DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
#
# Copyright (c) 2010 Oracle and/or its affiliates. All rights reserved.
#
# The contents of this file are subject to the terms of either the GNU
# General Public License Version 2 only ("GPL") or the Common Development
# and Distribution License("CDDL") (collectively, the "License").  You
# may not use this file except in compliance with the License.  You can
# obtain a copy of the License at
# https://glassfish.dev.java.net/public/CDDL+GPL_1_1.html
# or packager/legal/LICENSE.txt.  See the License for the specific
# language governing permissions and limitations under the License.
#
# When distributing the software, include this License Header Notice in each
# file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
#
# GPL Classpath Exception:
# Oracle designates this particular file as subject to the "Classpath"
# exception as provided by Oracle in the GPL Version 2 section of the License
# file that accompanied this code.
#
# Modifications:
# If applicable, add the following below the License Header, with the fields
# enclosed by brackets [] replaced by your own identifying information:
# "Portions Copyright [year] [name of copyright owner]"
#
# Contributor(s):
# If you wish your version of this file to be governed by only the CDDL or
# only the GPL Version 2, indicate your decision by adding "[Contributor]
# elects to include this software in this distribution under the [CDDL or GPL
# Version 2] license."  If you don't indicate a single choice of license, a
# recipient has the option to distribute your version of this file under
# either the CDDL, the GPL Version 2 or to extend the choice of license to
# its licensees as provided above.  However, if you add GPL Version 2 code
# and therefore, elected the GPL Version 2 license, then the option applies
# only if the new code is made subject to such option by the copyright
# holder.
#

import os;

class build_pkg (dict):
    def __init__(self, name, version):
        self.update({
            "name"          : name,
            "version"       : version,
            "dirs"          : {},
            "files"         : {},
            "links"         : {}
        })
    
    # add directories recursively, kind of like "mkdir -p"
    def mkdirs(self,path,attributes={}):
        head = ""
        for i in path.split('/'):
            if len(i)==0 :
                continue
            if len(head) > 0 :
                head += '/'
            head += i
            if head not in self["dirs"]:
                # print 'Adding %s' % head
                self["dirs"][head] = attributes

    # add a file, and create parent directories if necessary
    def addfile(self,fullpath,attributes={}):
        self.mkdirs(fullpath[0:fullpath.rindex('/')])
        self["files"][fullpath] = attributes

    def addlink(self,fullpath,target):
        self["links"][fullpath] = target

    # recursively add files and sub-directories in src, to be installed into the 'target' dir
    def addRecursively(self,target,src,filter=None):
        for root, dirs, files in os.walk(src):
            reldir = root[len(src)+1:]
            for name in files:
                if filter==None or filter(name):
                    self.addfile(os.path.join(target,reldir,name), {"file":os.path.join(root,name)})
            for name in dirs:
                if filter==None or filter(name):
                    self.mkdirs(os.path.join(target,reldir,name))

